/*	$LAAS: jmem.c 2008/08/21 15:40:10 mallet $ */

/*
 * Copyright (c) 2000-2006,2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <stdio.h>

#include "jpeglib.h"
#include "jerror.h"

/**
 ** Driver de sortie de pour libjpeg pour compression en memoire
 **/

typedef struct {
    struct jpeg_destination_mgr pub;	/* public fields */
    JOCTET *buffer;
    int size;
} my_destination_mgr;

typedef my_destination_mgr *my_dest_ptr;

/*
 * Initialize destination
 */
METHODDEF(void)
init_destination(j_compress_ptr cinfo)
{
    my_dest_ptr dest = (my_dest_ptr)cinfo->dest;

    dest->pub.next_output_byte = dest->buffer;
    dest->pub.free_in_buffer = dest->size;
}

/*
 * Empty the output buffer
 */
METHODDEF(boolean)
empty_output_buffer(j_compress_ptr cinfo)
{
    my_dest_ptr dest = (my_dest_ptr)cinfo->dest;

    dest->pub.next_output_byte = dest->buffer;
    dest->pub.free_in_buffer = dest->size;
    return TRUE;
}

/*
 * Terminate destination
 */
METHODDEF(void)
term_destination(j_compress_ptr cinfo)
{
    return;
}

/*
 * Prepare for output to a  memory buffer
 */
GLOBAL(void)
viman_jpeg_mem_dest(j_compress_ptr cinfo, unsigned char *buffer, int size)
{
    my_dest_ptr dest;

    if (cinfo->dest == NULL) {
	cinfo->dest = (struct jpeg_destination_mgr *)
		      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo,
						  JPOOL_PERMANENT,
						  sizeof(my_destination_mgr));
    }
  dest = (my_dest_ptr) cinfo->dest;
  dest->pub.init_destination = init_destination;
  dest->pub.empty_output_buffer = empty_output_buffer;
  dest->pub.term_destination = term_destination;
  dest->buffer = buffer;
  dest->size = size;
}

GLOBAL(int)
viman_jpeg_mem_size(j_compress_ptr cinfo)
{
    my_dest_ptr dest = (my_dest_ptr)cinfo->dest;

    if (dest) {
	return (dest->pub.next_output_byte - dest->buffer);
    } else {
	return -1;
    }
}
